/* rtune.c
 *
 * A command line interface for radio cards compliant with the Video4Linux
 * Specification
 *
 * by Fred Gleason <fredg@wava.com>
 */
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/videodev.h>

#define RTUNEVERSION "rtune - version 0.0.1"
#define RTUNEUSAGE "rtune [-i][-a <volume>][-m 0|1][-s <freq>] <radio-dev>"

/*
 * Function Prototypes
 */
void enumerate_card(int);
int set_freq(int,double,int);
int set_volume(int,int,int);
int set_mute(int,int,int);

/*
 * Global Variables
 */
static int debug=0;

int main(int argc,char *argv[])
{
  int i;
  int found=0;
  int infomode=0,setmode=0,volmode=0,achanmode=0,mutemode=0;
  int volume=0,mute=0,achan=0;
  int tuner;
  float freq;
  double frequency=0;

  /* Get Options */
  if(argc>1) {
    for(i=1;i<(argc-1);i++) {
      found=0;
      if(strcmp(argv[i],"-v")==0) {   /* Print Version Number */
	printf(RTUNEVERSION);
	printf("\n\n");
	exit(0);
	found=1;
      }
      if(strcmp(argv[i],"-d")==0) {   /* Debug Mode */
	debug=1;
	found=1;
	printf("rtune: debug mode...\n");
      }
      if(strcmp(argv[i],"-i")==0) {   /* Info Mode */
	infomode=1;
	found=1;
      }
      if(strcmp(argv[i],"-s")==0) {   /* Set Frequency Mode */
	setmode=1;
	found=1;
	if(sscanf(argv[++i],"%f",&freq)!=1) {
	  printf("rtune: invalid argument\n");
	  exit(1);
	}
	frequency=(double)freq;
      }
      if(strcmp(argv[i],"-a")==0) {   /* Set Volume Mode */
	volmode=1;
	found=1;
	if(sscanf(argv[++i],"%d",&volume)!=1) {
	  printf("rtune: invalid argument: %s\n",argv[i]);
	  exit(1);
	}
      }
      if(strcmp(argv[i],"-m")==0) {   /* Set Mute Mode */
	mutemode=1;
	found=1;
	if(sscanf(argv[++i],"%d",&mute)!=1) {
	  printf("rtune: invalid argument: %s\n",argv[i]);
	  exit(1);
	}
      }
      if(strcmp(argv[i],"-c")==0) {   /* Select Audio Channel Mode */
	achanmode=1;
	found=1;
	if(sscanf(argv[++i],"%d",&achan)!=1) {
	  printf("rtune: invalid argument: %s\n",argv[i]);
	  exit(1);
	}
      }
      if(found==0) {
	printf(RTUNEUSAGE);
	printf("\n\n");
	exit(1);
      }
    }
  } 
  else {
    printf(RTUNEUSAGE);
    printf("\n\n");
    exit(1);
  }  

  tuner=open(argv[argc-1],O_RDWR);
  if(tuner<0) {
    perror("rtune");
    exit(1);
  }

  if(infomode==1) {    /* Info Mode */
    enumerate_card(tuner);
  }
  if(setmode!=0) {     /* Set Frequency Mode */
    if(set_freq(tuner,frequency,setmode)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
  }
  if(volmode!=0) {     /* Set Volume Mode */
    if(set_volume(tuner,volume,achan)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
  }
  if(mutemode!=0) {     /* Set Mute Mode */
    if(set_mute(tuner,mute,achan)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
  }
  close(tuner);
  exit(0);
} 


int set_volume(int tuner,int volume,int achan)
{
  struct video_audio vaudio;

  vaudio.audio=achan;
  if(ioctl(tuner,VIDIOCGAUDIO,&vaudio)<0) {
    return -1;
  }
  vaudio.volume=volume;
  if(ioctl(tuner,VIDIOCSAUDIO,&vaudio)<0) {
    return -1;
  }
  printf("Volume: %d\n",volume);
  return 0;
}


int set_freq(int tuner,double frequency,int unit)
{
  int i;
  unsigned freq;
  struct video_capability vcap;
  struct video_tuner vtuner;

  /*
   * Find a tuner in the correct range
   */
  if(ioctl(tuner,VIDIOCGCAP,&vcap)<0) {
    return -1;
  }
  for(i=0;i<vcap.channels;i++) {
    vtuner.tuner=i;
    if(ioctl(tuner,VIDIOCGTUNER,&vtuner)<0) {
      return -1;
    }
    /* 
     * Normalize the frequency value
     */

    if((vtuner.flags&VIDEO_TUNER_LOW)==0) {
      freq=(unsigned)(frequency*16/1000);
    }
    else {
      freq=(unsigned)(frequency*16);
    }

    if(debug!=0) {
      printf("For tuner %d:\n",i);
      printf("  checking requested frequency of %d\n",freq/16);
      printf("  against LOW=%d and HIGH=%d\n",(int)(vtuner.rangelow/16),
	     (int)(vtuner.rangehigh/16));
    }
    if((vtuner.rangelow<=freq)&&(vtuner.rangehigh>=freq)) {
      /*
       * This is the right tuner
       */
      if(debug!=0) {
	printf("Tuner %d selected.\n",i);
      }
      if(ioctl(tuner,VIDIOCSTUNER,&i)<0) {
	return -1;
      }
      if(ioctl(tuner,VIDIOCSFREQ,&freq)<0) {
	return -1;
      }
      return 0;
    }
  }
  errno=EINVAL;
  return -1;
}


void enumerate_card(int tuner)
{
  int i;
  unsigned freq;
  struct video_capability vcap;
  struct video_tuner vtuner;
  struct video_audio vaudio;

  /*
   * Enumerate the card capabilities
   */
  if(ioctl(tuner,VIDIOCGCAP,&vcap)<0) {
    perror("rtune");
  }
  printf("\n");
  printf("  --- INTERFACE DATA ---\n");
  printf("Interface Name: %s\n",vcap.name);
  printf("  Number of Tuner Channels: %d\n",vcap.channels);
  printf("  Number of Audio Devices: %d\n",vcap.audios);
  printf("  Capability Flags:\n");
  if((vcap.type&VID_TYPE_CAPTURE)!=0) {
    printf("    VID_TYPE_CAPTURE\n");
  }
  if((vcap.type&VID_TYPE_TUNER)!=0) {
    printf("    VID_TYPE_TUNER\n");
  }
  if((vcap.type&VID_TYPE_TELETEXT)!=0) {
    printf("    VID_TYPE_TELETEXT\n");
  }
  if((vcap.type&VID_TYPE_OVERLAY)!=0) {
    printf("    VID_TYPE_OVERLAY\n");
  }
  if((vcap.type&VID_TYPE_CHROMAKEY)!=0) {
    printf("    VID_TYPE_CHROMAKEY\n");
  }
  if((vcap.type&VID_TYPE_CLIPPING)!=0) {
    printf("    VID_TYPE_CLIPPING\n");
  }
  if((vcap.type&VID_TYPE_FRAMERAM)!=0) {
    printf("    VID_TYPE_FRAMERAM\n");
  }
  if((vcap.type&VID_TYPE_SCALES)!=0) {
    printf("    VID_TYPE_SCALES\n");
  }
  if((vcap.type&VID_TYPE_MONOCHROME)!=0) {
    printf("    VID_TYPE_MONOCHROME\n");
  }
  if((vcap.type&VID_TYPE_SUBCAPTURE)!=0) {
    printf("    VID_TYPE_SUBCAPTURE\n");
  }
  printf("\n");
  
  /*
   * Enumerate the tuner capabilities
   */
  printf("    --- TUNER DATA ---\n");
  for(i=0;i<vcap.channels;i++) {
    if(ioctl(tuner,VIDIOCSTUNER,&i)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
    vtuner.tuner=i;
    if(ioctl(tuner,VIDIOCGTUNER,&vtuner)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
    printf("Tuner Name: %s\n",vtuner.name);
    printf("  Tuner Number: %d\n",vtuner.tuner);
    printf("  Tuner Low Frequency: %6.1f",((float)vtuner.rangelow)/16);
    if((vtuner.flags&VIDEO_TUNER_LOW)!=0) {
      printf(" kHz\n");
    }
    else {
      printf(" MHz\n");
    }
    printf("  Tuner High Frequency: %6.1f",((float)vtuner.rangehigh)/16);
    if((vtuner.flags&VIDEO_TUNER_LOW)!=0) {
      printf(" kHz\n");
    }
    else {
      printf(" MHz\n");
    }
    if(ioctl(tuner,VIDIOCGFREQ,&freq)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
    if((freq>=vtuner.rangelow)&&(freq<=vtuner.rangehigh)) {
      printf("  Current Frequency: %6.1f",((float)freq)/16);
      if((vtuner.flags&VIDEO_TUNER_LOW)!=0) {
	printf(" kHz\n");
      }
      else {
	printf(" MHz\n");
      }
    }
    printf("  Current Signal Strength: %d uV\n",vtuner.signal);
    printf("  Current Mode: ");
    if(VIDEO_MODE_PAL==vtuner.mode) {
      printf("PAL Video\n");
    }
    if(VIDEO_MODE_NTSC==vtuner.mode) {
      printf("NTSC Video\n");
    }
    if(VIDEO_MODE_SECAM==vtuner.mode) {
      printf("SECAM Video\n");
    }
    if(VIDEO_MODE_AUTO==vtuner.mode) {
      printf("AUTO or RADIO\n");
    }
    printf("  Capability Flags:\n");
    if((VIDEO_TUNER_PAL&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_PAL\n");
    }
    if((VIDEO_TUNER_NTSC&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_NTSC\n");
    }
    if((VIDEO_TUNER_SECAM&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_SECAM\n");
    }
    if((VIDEO_TUNER_LOW&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_LOW\n");
    }
    if((VIDEO_TUNER_NORM&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_NORM\n");
    }
    if((VIDEO_TUNER_STEREO_ON&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_STEREO_ON\n");
    }
    if((VIDEO_TUNER_RDS_ON&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_RDS_ON\n");
    }
    if((VIDEO_TUNER_MBS_ON&vtuner.flags)!=0) {
      printf("    VIDEO_TUNER_MBS_ON\n");
    }
  }
  printf("\n");

  /*
   * Enumerate the audio devices
   */
  printf("    --- AUDIO DATA ---\n");
  for(i=0;i<vcap.audios;i++) {
    if(ioctl(tuner,VIDIOCGAUDIO,&vaudio)<0) {
      perror("rtune");
      close(tuner);
      exit(1);
    }
    printf("Channel Name: %s\n",vaudio.name);
    printf("  Channel Number: %d\n",vaudio.audio);
    if((vaudio.flags&VIDEO_AUDIO_VOLUME)!=0) {
      printf("  Channel Volume: %d\n",vaudio.volume);
    }
    if((vaudio.flags&VIDEO_AUDIO_BASS)!=0) {
      printf("  Channel Bass: %d\n",vaudio.bass);
    }
    if((vaudio.flags&VIDEO_AUDIO_TREBLE)!=0) {
      printf("  Channel Treble: %d\n",vaudio.treble);
    }
    printf("  Channel Step-size: %d\n",vaudio.step);
    if((vaudio.flags&VIDEO_AUDIO_MUTABLE)!=0) {
      printf("  Audio is Mutable\n");
    }
    if((vaudio.flags&VIDEO_AUDIO_MUTE)!=0) {
      printf("  Audio is Muted\n");
    }
    if(vaudio.mode==VIDEO_SOUND_MONO) {
      printf("  Current Mode: MONO\n");
    }
    if(vaudio.mode==VIDEO_SOUND_STEREO) {
      printf("  Current Mode: STEREO\n");
    }
    if(vaudio.mode==VIDEO_SOUND_LANG1) {
      printf("  Current Mode: ALTERNATE LANGUAGE 1\n");
    }
    if(vaudio.mode==VIDEO_SOUND_LANG2) {
      printf("  Current Mode: ALTERNATE LANGUAGE 2\n");
    }
  }
}
  


int set_mute(int tuner,int mute,int achan)
{
  struct video_audio vaudio;

  vaudio.audio=achan;
  if(ioctl(tuner,VIDIOCGAUDIO,&vaudio)<0) {
    return -1;
  }
  if(mute==0) {
    vaudio.flags|=VIDEO_AUDIO_MUTE;
  }
  else {
    vaudio.flags&=(~VIDEO_AUDIO_MUTE);
  }
  if(ioctl(tuner,VIDIOCSAUDIO,&vaudio)<0) {
    return -1;
  }
  return 0;
}


